"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.configureStore = void 0;

var _react = _interopRequireDefault(require("react"));

var _mobxDevtoolsMst = _interopRequireDefault(require("mobx-devtools-mst"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * Generic Types:
 *  - first generic is GenRootStoreModel, which extends Instance type from MST. This
 *  is needed for methods, that Instance provides.
 *  - second generic is GenStoreEnv, which extends object for now. This is needed for TS to recognize
 *  stores that exist on the store bus
 *  - third generic is GenInstantiatedStoresBox, which represents instantiated stores
 *  that are loaded into the RootStore.
 *  - fourth generic is GenBus, which represent instantiated stores
 *  that must be accessible through the store bus
 */
const configureStore = ({
  RootStore,
  InstantiatedStoresBox,
  Bus
}) => {
  // for now it will be a function with its own scope
  const createStore = () => {
    // @ts-ignore
    const env = { ...Bus
    };
    const rootStore = RootStore.create({ ...InstantiatedStoresBox
    }, env);
    return rootStore;
  };

  const rootStore = createStore(); // React utils

  const StoreContext = /*#__PURE__*/_react.default.createContext(null);

  const {
    NODE_ENV
  } = process.env;

  const StoreProvider = ({
    children
  }) => {
    if (!children) {
      throw new Error('Provider cannot be mounted without React Element inside');
    }

    if (NODE_ENV !== 'production') {
      (0, _mobxDevtoolsMst.default)(rootStore);
    }

    return /*#__PURE__*/_react.default.createElement(StoreContext.Provider, {
      value: rootStore
    }, children);
  };

  const useStore = () => {
    const value = _react.default.useContext(StoreContext);

    if (!value) {
      throw new Error('useStore must be used within a StoreProvider. Please, add context!');
    }

    return value;
  }; // type MapStore<GenReturn> = (store: GenRootStoreModel) => GenReturn;


  /**
     * It is a simple property picker, same as
     * redux selector (memoization is handled by MST itself)
     * **NOTE!** Define a mapStore function beforehand.
     * Pass it to useInject, the latter will select a property from a global store
     * @example
      const mapStore = (
        store // or destructure it
      ) => {
        return {
            someDataForComponent: store.some.deep.property.we.need
        }
      };
      ...
      const {someDataForComponent} = useInject(mapStore);
     */
  const useInject = mapStore => {
    const store = useStore();
    return mapStore(store);
  };

  const storeStruct = {
    rootStore,
    StoreContext,
    StoreProvider,
    useStore,
    useInject
  };
  return storeStruct;
};

exports.configureStore = configureStore;